@@ -1,5 +1,6 @@
-// Copyright (c) 2001-2004 Rob Kaper <cap@capsi.com>,
-//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
+// Copyright © 2001-2004 Rob Kaper <cap@capsi.com>,
+//             2001 Erik Bourget <ebourg@cs.mcgill.ca>
+//             2010-2015 Sylvain Rochet <gradator@gradator.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -11,9 +12,8 @@
 // General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// along with this program; see the file COPYING. If not, see
+// <http://www.gnu.org/licenses/>.
 
 #ifndef MONOP_GAME_H
 #define MONOP_GAME_H
@@ -22,6 +22,7 @@
 #include <string>
 
 #include "gameobject.h"
+#include "display.h"
 
 class Auction;
 class Card;
@@ -49,8 +50,8 @@ class Game : public GameObject
 		void loadGameType(const std::string &gameType);
 
 		GameObject *findConfigOption(const std::string &name);
+		bool getBoolConfigOption(const std::string &name);
 		void editConfiguration(Player *pInput, char *data);
-		void editConfig(Player *pInput, char *data);
 		void start(Player *pInput);
 
 		void setTokenLocation(Player *pInput, unsigned int estateId);
@@ -60,20 +61,11 @@ class Game : public GameObject
 		void setAllClientsMoving(Estate *estate);
 
 		// Member management
-		void setStatus(Status status) { m_status = status; }
 		void setDescription(const std::string data);
 		void setHouses(int _h) { m_houses = _h; }
 		void setHotels(int _h) { m_hotels = _h; }
-		void setCollectFines(const bool collectFines);
-		void setAuctionsEnabled(const bool auctionsEnabled);
-		void setDoublePassMoney(bool doublePassMoney);
-		void setAlwaysShuffle(bool alwaysShuffle);
-		void setUnlimitedHouses(bool unlimitedHouses);
-		void setNoRentInJail(bool noRentInJail);
-		void setAllowEstateSales(bool allowEstateSales);
 
 		Status status() { return m_status; }
-		const std::string statusLabel();
 		Player *master() { return m_master; }
 		Estate *goEstate() { return m_goEstate; }
 		const std::string gameType() { return m_gameType; }
@@ -85,9 +77,8 @@ class Game : public GameObject
 		bool sendChildXMLUpdate(Player *pOutput, bool emptyUpdate);
 		void unsetChildProperties();
 
-		int players();
-		unsigned int connectedPlayers();
-		Player *addPlayer(Player *, const bool &isMaster = false, const bool &isSpectator = false);
+		int players(bool includeSpectators = false);
+		Player *addPlayer(Player *, const bool &isSpectator = false, const bool &isFirst = false);
 		Player *findPlayer(int playerId);
 		Player *kickPlayer(Player *player, int playerId);
 		void upgradePlayer(Player *player, int playerId);
@@ -95,16 +86,17 @@ class Game : public GameObject
 
 		void updateTurn();
 		bool landPlayer(Player *pTurn, const bool directMove, const std::string &rentmath = "");
-		bool giveCard(Player *player, Estate *estate, Card *card);
+		bool giveCard(Player *player, Card *card);
 		void declareBankrupt(Player *player);
 		void bankruptPlayer(Player *player);
-		void setDisplay(Estate *estate, bool clearText, bool clearButtons, const char *data, ...);
+		void sendDisplayMsg(Display *display, Player *except = NULL);
 		void sendMsgEstateUpdate(Estate *);
-		void sendStatus(Player *player);
+		void sendStatus(Status status, Player *player = NULL);
 		void sendEstateList(Player *player);
 		void sendEstateGroupList(Player *player);
-		void sendPlayerList(Player *player, bool includeCards = false);
-		void sendFullUpdate(Player *player, bool userRequest = false);
+		void sendPlayerList(Player *player);
+		void sendCardList(Player *pOut);
+		void sendFullUpdate(Player *player);
 
 		int debts();
 		void newDebtToAll(Player *from, int amount);
@@ -119,15 +111,15 @@ class Game : public GameObject
 		Auction *auction();
 		// Returns 0 on successful bid, 1 on error.
 		int bidInAuction(Player *pInput, char *data);
-		void delAuction();
 
 		void newTrade(Player *pInput, unsigned int playerId);
 		Trade *findTrade(Player *player);
 		Trade *findTrade(int tradeId);
 		void delTrade(Trade *trade);
 		void acceptTrade(Player *pInput, char *);
-		void rejectTrade(Player *pInput, unsigned int tradeId);
+		void rejectTrade(Player *pInput, unsigned int tradeId, const bool verbose = true);
 		void completeTrade(Trade *trade);
+		void abortAuction();
 		void completeAuction();
 
 		EstateGroup *newGroup(const std::string &name);
@@ -140,7 +132,6 @@ class Game : public GameObject
 		unsigned int estates() const { return m_estates.size(); }
 		void setPausedForDialog(const bool paused);
 		bool pausedForDialog();
-		void delEstate(Estate *);
 		void transferEstate(Estate *estate, Player *player, const bool verbose = false);
 		CardGroup *newCardGroup(const std::string name);
 		CardGroup *findCardGroup(const std::string name);
@@ -149,7 +140,6 @@ class Game : public GameObject
 		void transferCard(Card *card, Player *player, const bool verbose = false);
 		void transferObject(const enum GameObject::Type, unsigned int id, Player *player, const bool verbose = false);
 		void removePlayer(Player *);
-		int lowestFreeId();
 		int totalAssets();
 		void rollDice();
 		int dice[2];
