@@ -1,5 +1,6 @@
-// Copyright (c) 2001-2004 Rob Kaper <cap@capsi.com>,
-//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
+// Copyright © 2001-2004 Rob Kaper <cap@capsi.com>,
+//             2001 Erik Bourget <ebourg@cs.mcgill.ca>
+//             2010-2015 Sylvain Rochet <gradator@gradator.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -11,9 +12,8 @@
 // General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// along with this program; see the file COPYING. If not, see
+// <http://www.gnu.org/licenses/>.
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -31,30 +31,30 @@
 #include "player.h"
 #include "trade.h" // move trade commands to game/engine
 
-Player::Player(Socket *socket, int id) : GameObject(id, GameObject::GPlayer)
+Player::Player(Socket *socket) : GameObject(-1, GameObject::GPlayer)
 {
 	m_socket = socket;
 	m_game = 0;
-	m_display = new Display();
-	setProperty("cookie", std::string(itoa(id)) + "/" + itoa(rand()), this);
-	reset();
+	m_estate = 0;
+	m_destination = 0;
+	m_tokenLocation = 0;
+	m_identified = false;
 }
 
 Player::~Player()
 {
-    while (!m_cards.empty()) { delete *m_cards.begin(); m_cards.erase(m_cards.begin()); }
-
-    delete m_display;
+	while (!m_cards.empty()) { delete *m_cards.begin(); m_cards.erase(m_cards.begin()); }
+	while (!m_display.empty()) { delete *m_display.begin(); m_display.erase(m_display.begin()); }
 }
 
 void Player::reset(bool removeProperties)
 {
-    while (!m_cards.empty()) { delete *m_cards.begin(); m_cards.erase(m_cards.begin()); }
+	while (!m_cards.empty()) { delete *m_cards.begin(); m_cards.erase(m_cards.begin()); }
+	while (!m_display.empty()) { delete *m_display.begin(); m_display.erase(m_display.begin()); }
 
 	m_estate = m_destination = m_tokenLocation = 0;
 
-	if (removeProperties)
-	{
+	if (removeProperties) {
 		removeProperty("money");
 		removeProperty("doublecount");
 		removeProperty("jailcount");
@@ -68,12 +68,14 @@ void Player::reset(bool removeProperties)
 		removeProperty("canauction");
 		removeProperty("location");
 		removeProperty("directmove");
-		removeProperty("master"); // FIXME: DEPRECATED 1.0
+		removeProperty("turnorder");
 	}
-	m_requestedUpdate = false;
+}
 
-	m_display->setEstate(0);
-	m_display->resetButtons();
+void Player::identify(int id) {
+	m_id = id;
+	setProperty("cookie", std::string(itoa(m_id)) + "/" + itoa(rand()), this);
+	m_identified = true;
 }
 
 void Player::ioWrite(const char *fmt, ...)
@@ -159,70 +161,47 @@ void Player::ioNoSuchCmd(const std::string data)
 		ioError("No such command.");
 }
 
-void Player::setDisplay(Estate *estate)
+void Player::sendDisplay(Display *display)
 {
-	m_display->setEstate(estate);
-	m_display->resetButtons();
-}
+	std::string text, estate;
+	if (display->text().size()) {
+		text = "text=\"" + display->text() + "\" ";
+	}
 
-void Player::setDisplay(Estate *estate, const char *data, ...)
-{
-	va_list arg;
-	char buf[2048];
+	if (display->clearEstate()) {
+		estate = "estateid=\"-1\" ";
+	} else if (display->estate()) {
+		estate = "estateid=\"" + itoa(display->estate()->id()) + "\" ";
+	}
 
-	va_start(arg, data);
-	vsnprintf(buf, sizeof(buf)-1, data, arg);
-	va_end(arg);
-	buf[sizeof(buf)-1] = 0;
+	ioWrite("<monopd><display %s%scleartext=\"%d\" clearbuttons=\"%d\"", estate.c_str(), text.c_str(), display->clearText(), display->clearButtons());
 
-	setDisplay(estate);
-	m_display->setText(std::string(buf));
-	m_display->resetButtons();
-}
-
-void Player::setDisplay(Estate *estate, const std::string data)
-{
-	setDisplay(estate);
-	m_display->setText(data);
-}
-
-void Player::setDisplayClearText(bool clearText)
-{
-	m_display->setClearText(clearText);
-}
-
-void Player::setDisplayClearButtons(bool clearButtons)
-{
-	m_display->setClearButtons(clearButtons);
-}
-
-void Player::addDisplayButton(const std::string command, const std::string caption, const bool enabled)
-{
-	m_display->addButton(command, caption, enabled);
-}
-
-void Player::sendDisplayMsg()
-{
-	Estate *estate = m_display->estate();
-	if (estate)
-	{
-		std::vector<DisplayButton *> buttons = m_display->buttons();
-		if (buttons.size() > 0)
-		{
-			ioWrite("<monopd><display estateid=\"%d\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\">", estate->id(), m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());
-			DisplayButton *button = 0;
-			for (std::vector<DisplayButton *>::iterator it = buttons.begin() ; it != buttons.end() && (button = *it) ; ++it)
-				ioWrite("<button command=\"%s\" caption=\"%s\" enabled=\"%d\"/>", button->command().c_str(), button->caption().c_str(), button->enabled());
-			ioWrite("</display></monopd>\n", estate->id(), m_display->text().c_str());
-		}
-		else
-			ioWrite("<monopd><display estateid=\"%d\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\"/></monopd>\n", estate->id(), m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());
+	std::vector<DisplayButton *> buttons = display->buttons();
+	if (buttons.size() > 0) {
+		ioWrite(">");
+		DisplayButton *button;
+		for (std::vector<DisplayButton *>::iterator it = buttons.begin() ; it != buttons.end() && (button = *it) ; ++it)
+			ioWrite("<button command=\"%s\" caption=\"%s\" enabled=\"%d\"/>", button->command().c_str(), button->caption().c_str(), button->enabled());
+		ioWrite("</display></monopd>\n");
 	}
 	else
-		ioWrite("<monopd><display estateid=\"-1\" text=\"%s\" cleartext=\"%d\" clearbuttons=\"%d\"/></monopd>\n", m_display->text().c_str(), m_display->clearText(), m_display->clearButtons());
+		ioWrite("/></monopd>\n");
+}
 
-	m_display->setClearText(false);
-	m_display->setClearButtons(false);
+void Player::sendDisplayMsg(Display *display)
+{
+	sendDisplay(display);
+	if (display->clearText()) {
+		while (!m_display.empty()) { delete *m_display.begin(); m_display.erase(m_display.begin()); }
+	}
+	m_display.push_back(new Display(display));
+}
+
+void Player::sendDisplayHistory()
+{
+	for(std::vector<Display *>::iterator it = m_display.begin(); it != m_display.end(); ++it) {
+		sendDisplay(*it);
+	}
 }
 
 void Player::sendClientMsg()
@@ -251,7 +230,14 @@ void Player::rollDice()
 	}
 
 	m_game->rollDice();
-	m_game->setDisplay(0, true, false, "%s rolls %d and %d.", getStringProperty("name").c_str(), m_game->dice[0], m_game->dice[1]);
+
+	Display display;
+	display.setText("%s rolls %d and %d.", getStringProperty("name").c_str(), m_game->dice[0], m_game->dice[1]);
+	display.resetText();
+	display.resetEstate();
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove Roll button */
+	sendDisplayMsg(&display);
 
 	// Take away many privileges
 	setBoolProperty("can_roll", false);
@@ -267,7 +253,9 @@ void Player::rollDice()
 			setBoolProperty("canrollagain", true);
 		else
 		{
-			m_game->setDisplay(m_estate, false, false, "%s throws three doubles and is thrown into jail.", getStringProperty("name").c_str());
+			Display display;
+			display.setText("%s throws three doubles and is thrown into jail.", getStringProperty("name").c_str());
+			m_game->sendDisplayMsg(&display);
 
 			if (Estate *eJail = m_game->findNextJailEstate(m_estate))
 				toJail(eJail);
@@ -290,6 +278,9 @@ void Player::rollDice()
 
 void Player::endTurn(bool userRequest)
 {
+	if (m_game->status() == Game::End)
+		return;
+
 	if (!getBoolProperty("hasturn"))
 		return;
 
@@ -302,7 +293,7 @@ void Player::endTurn(bool userRequest)
 
 	if (getBoolProperty("can_buyestate"))
 	{
-		if (m_game->getBoolProperty("auctionsenabled"))
+		if (m_game->getBoolConfigOption("auctionsenabled"))
 		{
 			if (userRequest)
 				ioError("You cannot end your turn, you must either buy or auction the property you are on.");
@@ -313,7 +304,11 @@ void Player::endTurn(bool userRequest)
 	}
 	else if (getBoolProperty("canrollagain"))
 	{
-		m_game->setDisplay(m_estate, false, false, "%s may roll again.", getStringProperty("name").c_str());
+		Display display;
+		display.setText("%s may roll again.", getStringProperty("name").c_str());
+		m_game->sendDisplayMsg(&display, this);
+		display.addButton(".r", "Roll", true);
+		sendDisplayMsg(&display);
 
 		setBoolProperty("can_roll", true);
 		setBoolProperty("canrollagain", false);
@@ -339,10 +334,10 @@ void Player::endTurn(bool userRequest)
 		return;
 	}
 
-	if (userRequest && m_display)
-	{
-		m_display->resetButtons();
-		sendDisplayMsg();
+	if (userRequest) {
+		Display display;
+		display.resetButtons(); /* Remove buy estate buttons: Buy, Auction, End Turn */
+		sendDisplayMsg(&display);
 	}
 
 	// Turn goes to next player
@@ -357,7 +352,7 @@ void Player::payJail()
 		ioError("Leaving jail costs %d, you only have %d.", payAmount, getIntProperty("money"));
 		return;
 	}
-	else if (m_game->getBoolProperty("collectfines"))
+	else if (m_game->getBoolConfigOption("collectfines"))
 	{
 		Estate * const ePayTarget = m_estate->payTarget();
 		if (ePayTarget)
@@ -368,7 +363,13 @@ void Player::payJail()
 	setBoolProperty("can_roll", true);
 	setBoolProperty("canusecard", false);
 	setProperty("jailcount", 0);
-	m_game->setDisplay(m_estate, false, true, "%s paid %d and has left jail, can roll now.", getStringProperty("name").c_str(), payAmount);
+
+	Display display;
+	display.setText("%s paid %d and has left jail, can roll now.", getStringProperty("name").c_str(), payAmount);
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove jail buttons: Pay, Use card, Roll */
+	display.addButton(".r", "Roll", true);
+	sendDisplayMsg(&display);
 }
 
 void Player::rollJail()
@@ -379,7 +380,12 @@ void Player::rollJail()
 	if (m_game->dice[0] == m_game->dice[1])
 	{
 		setBoolProperty("jailed", false);
-		m_game->setDisplay(m_estate, false, true, "Doubles, leaving jail!");
+
+		Display display;
+		display.setText("Doubles, leaving jail!");
+		m_game->sendDisplayMsg(&display, this);
+		display.resetButtons(); /* Remove jail buttons: Pay, Use card, Roll */
+		sendDisplayMsg(&display);
 
 		advance(m_game->dice[0] + m_game->dice[1], false);
 		return;
@@ -389,7 +395,12 @@ void Player::rollJail()
 	int jailCount = getIntProperty("jailcount");
 	if(jailCount < 3)
 	{
-		m_game->setDisplay(m_estate, false, true, "No doubles, staying in jail.");
+		Display display;
+		display.setText("No doubles, staying in jail.");
+		m_game->sendDisplayMsg(&display, this);
+		display.resetButtons(); /* Remove jail buttons: Pay, Use card, Roll */
+		sendDisplayMsg(&display);
+
 		m_game->updateTurn();
 		setProperty("jailcount", jailCount+1);
 		return;
@@ -400,18 +411,32 @@ void Player::rollJail()
 	setBoolProperty("jailed", false);
 	setBoolProperty("canusecard", false);
 	setProperty("jailcount", 0);
-	m_game->setDisplay(0, false, true, "No doubles, %s must pay %d now and leave jail.", getStringProperty("name").c_str(), payAmount);
+
+	Display display;
+	display.setText("No doubles, %s must pay %d now and leave jail.", getStringProperty("name").c_str(), payAmount);
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove jail buttons: Pay, Use card, Roll */
+	sendDisplayMsg(&display);
+
 	Estate * const ePayTarget = m_estate->payTarget();
 	if (!payMoney(payAmount))
 	{
-		m_game->setDisplay(m_estate, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", getStringProperty("name").c_str(), payAmount, (payAmount - getIntProperty("money")));
+		Display display;
+		display.setText("Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", getStringProperty("name").c_str(), payAmount, (payAmount - getIntProperty("money")));
+		m_game->sendDisplayMsg(&display);
+
 		setBoolProperty("canrollagain", true); // player can roll once debt is resolved
 		m_game->newDebt(this, 0, ePayTarget, payAmount);
 		return;
 	}
 
-	if (ePayTarget && m_game->getBoolProperty("collectfines"))
+	if (ePayTarget && m_game->getBoolConfigOption("collectfines"))
 		ePayTarget->addMoney(payAmount);
+
+	Display display2;
+	display2.addButton(".r", "Roll", true);
+	sendDisplayMsg(&display2);
+
 	setBoolProperty("can_roll", true);
 }
 
@@ -429,7 +454,13 @@ void Player::useJailCard()
 	setBoolProperty("jailed", false);
 	setBoolProperty("canusecard", false);
 	setProperty("jailcount", 0);
-	m_game->setDisplay(0, false, true, "%s used card and has left jail, can roll now.", getStringProperty("name").c_str());
+
+	Display display;
+	display.setText("%s used card and has left jail, can roll now.", getStringProperty("name").c_str());
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove jail buttons: Pay, Use card, Roll */
+	display.addButton(".r", "Roll", true);
+	sendDisplayMsg(&display);
 }
 
 void Player::buyEstate()
@@ -447,7 +478,13 @@ void Player::buyEstate()
 	}
 
 	m_game->transferEstate(m_estate, this);
-	m_game->setDisplay(m_estate, false, true, "Purchased by %s for %d.", getStringProperty("name").c_str(), m_estate->price());
+
+	Display display;
+	display.setText("Purchased by %s for %d.", getStringProperty("name").c_str(), m_estate->price());
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove buy estate buttons: Buy, Auction, End Turn */
+	sendDisplayMsg(&display);
+
 	setBoolProperty("can_buyestate", false);
 	setBoolProperty("canauction", false);
 	endTurn();
@@ -455,7 +492,7 @@ void Player::buyEstate()
 
 void Player::sellEstate(int estateId)
 {
-	if (!m_game->getBoolProperty("allowestatesales"))
+	if (!m_game->getBoolConfigOption("allowestatesales"))
 	{
 		ioError("Selling estates has been disabled in the configuration.");
 		return;
@@ -485,7 +522,10 @@ void Player::sellEstate(int estateId)
 
 	addMoney(sellPrice);
 	m_game->transferEstate(estate, 0);
-	m_game->setDisplay(estate, false, false, "%s sold %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), sellPrice);
+
+	Display display;
+	display.setText("%s sold %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), sellPrice);
+	m_game->sendDisplayMsg(&display);
 
 	if (getBoolProperty("hasdebt"))
 		m_game->solveDebts(this);
@@ -515,7 +555,10 @@ void Player::mortgageEstate(int estateId)
 		}
 		estate->setBoolProperty("mortgaged", false);
 		m_game->sendMsgEstateUpdate(estate);
-		m_game->setDisplay(estate, false, false, "%s unmortgaged %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("unmortgageprice"));
+
+		Display display;
+		display.setText("%s unmortgaged %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("unmortgageprice"));
+		m_game->sendDisplayMsg(&display);
 	}
 	else
 	{
@@ -531,7 +574,10 @@ void Player::mortgageEstate(int estateId)
 		m_game->sendMsgEstateUpdate(estate);
 
 		addMoney((estate->getIntProperty("mortgageprice")));
-		m_game->setDisplay(estate, false, false, "%s mortgaged %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("mortgageprice"));
+
+		Display display;
+		display.setText("%s mortgaged %s for %d.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("mortgageprice"));
+		m_game->sendDisplayMsg(&display);
 
 		if (getBoolProperty("hasdebt"))
 			m_game->solveDebts(this);
@@ -551,17 +597,24 @@ void Player::payTax(const bool percentage)
 	else
 		payAmount = m_estate->getIntProperty("tax");
 
-	m_game->setDisplay(m_estate, false, true, "%s chooses to pay the %s tax, which amounts to %d.", getStringProperty("name").c_str(), (percentage ? "assets based" : "static"), payAmount);
+	Display display;
+	display.setText("%s chooses to pay the %s tax, which amounts to %d.", getStringProperty("name").c_str(), (percentage ? "assets based" : "static"), payAmount);
+	m_game->sendDisplayMsg(&display, this);
+	display.resetButtons(); /* Remove pay tax buttons: Pay 200, Pay 10 Percent */
+	sendDisplayMsg(&display);
 
 	Estate * const ePayTarget = m_estate->payTarget();
 	if (!payMoney(payAmount))
 	{
-		m_game->setDisplay(m_estate, false, false, "Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", getStringProperty("name").c_str(), payAmount, (payAmount - getIntProperty("money")));
+		Display display;
+		display.setText("Game paused, %s owes %d but is not solvent. Player needs to raise %d in cash first.", getStringProperty("name").c_str(), payAmount, (payAmount - getIntProperty("money")));
+		m_game->sendDisplayMsg(&display);
+
 		m_game->newDebt(this, 0, ePayTarget, payAmount);
 		return;
 	}
 
-	if (ePayTarget && m_game->getBoolProperty("collectfines"))
+	if (ePayTarget && m_game->getBoolConfigOption("collectfines"))
 		ePayTarget->addMoney(payAmount);
 
 	endTurn();
@@ -607,7 +660,7 @@ void Player::buyHouse(int estateId)
 		return;
 	}
 
-	bool unlimitedHouses = m_game->getBoolProperty("unlimitedhouses");
+	bool unlimitedHouses = m_game->getBoolConfigOption("unlimitedhouses");
 
 	if (!unlimitedHouses)
 	{
@@ -642,13 +695,19 @@ void Player::buyHouse(int estateId)
 			m_game->setHotels(m_game->hotels() - 1);
 			m_game->setHouses(m_game->houses() + 4);
 		}
-		m_game->setDisplay(estate, false, false, "%s buys a hotel for %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+
+		Display display;
+		display.setText("%s buys a hotel for %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+		m_game->sendDisplayMsg(&display);
 	}
 	else
 	{
 		if (!unlimitedHouses)
 			m_game->setHouses(m_game->houses() - 1);
-		m_game->setDisplay(estate, false, false, "%s buys a house for %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+
+		Display display;
+		display.setText("%s buys a house for %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+		m_game->sendDisplayMsg(&display);
 	}
 
 	m_game->sendMsgEstateUpdate(estate);
@@ -679,7 +738,7 @@ void Player::sellHouse(int estateId)
 		return;
 	}
 
-	bool unlimitedHouses = m_game->getBoolProperty("unlimitedhouses");
+	bool unlimitedHouses = m_game->getBoolConfigOption("unlimitedhouses");
 	if ( !unlimitedHouses && (estate->getIntProperty("houses") == 5) && (m_game->houses() < 4) )
 	{
 		ioError("Hotel cannot be sold, Bank only has %d houses left.", m_game->houses());
@@ -696,13 +755,19 @@ void Player::sellHouse(int estateId)
 			m_game->setHotels(m_game->hotels() + 1);
 			m_game->setHouses(m_game->houses() - 4);
 		}
-		m_game->setDisplay(estate, false, false, "%s sells a hotel from %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+
+		Display display;
+		display.setText("%s sells a hotel from %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+		m_game->sendDisplayMsg(&display);
 	}
 	else
 	{
 		if (!unlimitedHouses)
 			m_game->setHouses(m_game->houses() + 1);
-		m_game->setDisplay(estate, false, false, "%s sells a house from %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+
+		Display display;
+		display.setText("%s sells a house from %s.", getStringProperty("name").c_str(), estate->getStringProperty("name").c_str());
+		m_game->sendDisplayMsg(&display);
 	}
 
 	m_game->sendMsgEstateUpdate(estate);
@@ -785,6 +850,16 @@ void Player::updateTradeObject(char *data)
 
 	// NULL player allowed, Trade will delete the component.
 	Player *playerTo = m_game->findPlayer(playerId);
+	if (playerTo) {
+		if (playerTo->getBoolProperty("bankrupt")) {
+			ioError("You cannot trade with bankrupt players.");
+			return;
+		}
+		if (playerTo->getBoolProperty("spectator")) {
+			ioError("You cannot trade with spectators.");
+			return;
+		}
+	}
 
 	trade->updateObject(object, pFrom, playerTo);
 }
@@ -834,12 +909,28 @@ void Player::updateTradeMoney(char *data)
 		ioError("No such playerFromId: %d.", playerFromId);
 		return;
 	}
+	if (pFrom->getBoolProperty("bankrupt")) {
+		ioError("You cannot trade with bankrupt players.");
+		return;
+	}
+	if (pFrom->getBoolProperty("spectator")) {
+		ioError("You cannot trade with spectators.");
+		return;
+	}
 	Player *pTo = m_game->findPlayer(playerToId);
 	if (!pTo)
 	{
 		ioError("No such playerToId: %d.", playerToId);
 		return;
 	}
+	if (pTo->getBoolProperty("bankrupt")) {
+		ioError("You cannot trade with bankrupt players.");
+		return;
+	}
+	if (pTo->getBoolProperty("spectator")) {
+		ioError("You cannot trade with spectators.");
+		return;
+	}
 
 	// TODO: should also count money player owes in other trade components
 	if(pFrom->getIntProperty("money") < money)
@@ -911,16 +1002,11 @@ bool Player::payMoney(int amount)
 	return true;
 }
 
-void Player::closeSocket()
-{
-	m_socket->setStatus(Socket::Close);
-}
-
 void Player::setSocket(Socket *socket)
 {
 	m_socket = socket;
 	if (m_socket)
-		setProperty( "host", socket->fqdn() );
+		setProperty( "host", socket->ipAddr() );
 	else
 		setProperty( "host", "" );
 }
@@ -935,15 +1021,28 @@ void Player::setTurn(const bool &turn)
 	{
 		if (getBoolProperty("jailed"))
 		{
-			setDisplay(m_estate, "You are in jail, turn %d.", getIntProperty("jailcount"));
-			addDisplayButton(".jp", "Pay", true);
-			addDisplayButton(".jc", "Use card", findOutOfJailCard());
+			Display display1;
+			display1.setText("%s is in jail, turn %d.", name().c_str(), getIntProperty("jailcount"));
+			display1.setEstate(m_estate);
+			m_game->sendDisplayMsg(&display1, this);
+
+			Display display2;
+			display2.setText("You are in jail, turn %d.", getIntProperty("jailcount"));
+			display2.setEstate(m_estate);
+			display2.addButton(".jp", "Pay", true);
+			display2.addButton(".jc", "Use card", findOutOfJailCard());
+			display2.addButton(".jr", "Roll", true);
+			sendDisplayMsg(&display2);
+
 			setBoolProperty("canusecard", findOutOfJailCard());
-			addDisplayButton(".jr", "Roll", true);
-			sendDisplayMsg();
 		}
-		else
+		else {
+			Display display;
+			display.addButton(".r", "Roll", true);
+			sendDisplayMsg(&display);
+
 			setBoolProperty("can_roll", true);
+		}
 	}
 	else
 	{
